@@ -15,7 +15,7 @@ import hipopybind
 #----------------------------------------------------------------------#
 # Basic  I/O behaviors
 
-def open(filename,mode="r"):
+def open(filename,mode="r",tags=None):
     """
     Parameters
     ----------
@@ -23,16 +23,20 @@ def open(filename,mode="r"):
     mode : string, optional
         File mode ("r" : read, "w" : write, "a" : append)
         Default : "r"
+    tags : int or list of ints, optional
+        Set bank tags for reader to use.  0 works for most banks.
+        1 is needed for scaler banks.
+        Default : None
 
     Description
     -----------
     Open a HIPO file to read.
     """
-    f = hipofile(filename,mode=mode)
+    f = hipofile(filename,mode=mode,tags=tags)
     f.open()
     return f
 
-def iterate(files,banks=None,step=100):
+def iterate(files,banks=None,step=100,tags=None):
     """
     Parameters
     ----------
@@ -41,16 +45,20 @@ def iterate(files,banks=None,step=100):
     banks : list, optional
         List of bank names to read
         Default : None
-    step : int
+    step : int, optional
         Batch size for iterating through file events
         Default : 100
+    tags : int or list of ints, optional
+        Set bank tags for reader to use.  0 works for most banks.
+        1 is needed for scaler banks.
+        Default : None
 
     Description
     -----------
     Iterate through a list of hipofiles reading all banks unless specific banks are specified.
     Iteration is broken into batches of step events.
     """
-    f = hipochain(files,banks,step=step)
+    f = hipochain(files,banks,step=step,tags=tags)
     return f
 
 def create(filename):
@@ -90,15 +98,13 @@ class hipofile:
     ----------
     filename : string
         Full path name of HIPO file
-    libpath : string
-        Path to shared library file
     mode : string
         File mode ("r" : read, "a" : append, "w" : write)
     reader : hipopybind.Reader
         HIPO file reader
     writer : hipopybind.Writer
         HIPO file writer
-    dict : hipopybind.Dictionary
+    dictionary : hipopybind.Dictionary
         HIPO file schema dictionary
     event : hipopybind.Event
         HIPO event for reading and writing banks
@@ -149,7 +155,7 @@ class hipofile:
     # getShorts
     # getLongs
 
-    def __init__(self,filename,mode="r"):
+    def __init__(self,filename,mode="r",tags=None):
         """
         Parameters
         ----------
@@ -171,6 +177,7 @@ class hipofile:
         self.buffext    = "~"
         self.buffname   = None
         self.banklist   = {}
+        self.tags       = tags
         
     def open(self):
         """
@@ -179,7 +186,9 @@ class hipofile:
         Open a HIPO file to read, write (from scratch), or append data.
         IMPORTANT:  Make sure you add schema before opening a file to write!
         """
+
         if self.mode=="r":
+            if self.tags is not None: self.reader.setTags(self.tags) #NOTE: Only set tags for files to read since not yet tested for writing files.
             self.reader.open(self.filename)
             self.reader.readDictionary(self.dictionary)
         elif self.mode=="w": self.writer.open(self.filename)
@@ -801,12 +810,13 @@ class hipofileIterator:
                 self.hipofile.event.getStructure(self.hipofile.banklist[bank])#NOTE: NECESSARY OR YOU WILL NOT READ ANY DATA!
                 for item in self.items[bank]:
                     data = []
-                    if   self.items[bank][item]=="D": data = self.hipofile.getDoubles(bank,item)
-                    elif self.items[bank][item]=="I": data = self.hipofile.getInts(bank,item)
-                    elif self.items[bank][item]=="F": data = self.hipofile.getFloats(bank,item)
-                    elif self.items[bank][item]=="L": data = self.hipofile.getLongs(bank,item)
-                    elif self.items[bank][item]=="S": data = self.hipofile.getShorts(bank,item)
-                    elif self.items[bank][item]=="B": data = self.hipofile.getBytes(bank,item)
+                    item_type = self.items[bank][item]
+                    if   item_type=="D": data = self.hipofile.getDoubles(bank,item)
+                    elif item_type=="I": data = self.hipofile.getInts(bank,item)
+                    elif item_type=="F": data = self.hipofile.getFloats(bank,item)
+                    elif item_type=="L": data = self.hipofile.getLongs(bank,item)
+                    elif item_type=="S": data = self.hipofile.getShorts(bank,item)
+                    elif item_type=="B": data = self.hipofile.getBytes(bank,item)
 
                     # Add bank data to event dictionary
                     event[bank+"_"+item] = [data]
@@ -829,13 +839,16 @@ class hipochain:
         Currently fixed to always be in read mode ("r")
     verbose : boolean
         Currently fixed to always be False
+    tags : int or list of ints
+        Set bank tags for reader to use.  0 works for most banks.
+        1 is needed for scaler banks.
 
     Description
     -----------
     Chains files together so they may be read continuously.
     """
 
-    def __init__(self,names,banks=None,step=100,mode="r"):
+    def __init__(self,names,banks=None,step=100,mode="r",tags=None):
         self.names   = names
 
         # Parse regex NOTE: Must be full or relative path from $PWD.  ~/... does not work.
@@ -854,6 +867,7 @@ class hipochain:
         self.step    = step
         self.mode    = "r"   #TODO: Does it make sense to just fix this?
         self.verbose = False #TODO: Do we really need this?
+        self.tags    = tags
 
     def __iter__(self):
         return hipochainIterator(self)
@@ -887,6 +901,7 @@ class hipochainIterator:
 
     def __init__(self,chain):
         self.chain   = chain
+        self.nnames  = len(self.chain.names) #NOTE: Assumes this will stay constant.
         self.idx     = -1
         self.counter = 0
         self.file    = None
@@ -901,8 +916,8 @@ class hipochainIterator:
         """
         # Open file
         self.idx += 1 #NOTE: Do this before everything below since we initiate at -1.
-        if (self.idx>=len(self.chain.names)): return #NOTE: Sanity check
-        self.file = hipofile(self.chain.names[self.idx],mode=self.chain.mode)
+        if self.idx>=self.nnames: return #NOTE: Sanity check
+        self.file = hipofile(self.chain.names[self.idx],mode=self.chain.mode,tags=self.chain.tags)
         self.file.open()
         
         if self.chain.banks is None: self.chain.banks = self.file.getBanks() #NOTE: This assumes all the files in the chain have the same banks.
@@ -922,7 +937,7 @@ class hipochainIterator:
 
         if self.idx == -1: self.switchFile() # Load first file manually
 
-        if self.idx<(len(self.chain.names)): #TODO: Check this condition.
+        if self.idx<(self.nnames): #TODO: Check this condition.
 
             # Check if output array has been initialized
             if self.dict is None:
@@ -936,16 +951,17 @@ class hipochainIterator:
                     self.file.event.getStructure(self.file.banklist[bank])#NOTE: NECESSARY OR YOU WILL NOT READ ANY DATA!
                     for item in self.items[bank]:
                         data = []
-                        if   self.items[bank][item]=="D": data = self.file.getDoubles(bank,item)
-                        elif self.items[bank][item]=="I": data = self.file.getInts(bank,item)
-                        elif self.items[bank][item]=="F": data = self.file.getFloats(bank,item)
-                        elif self.items[bank][item]=="L": data = self.file.getLongs(bank,item)
-                        elif self.items[bank][item]=="S": data = self.file.getShorts(bank,item)
-                        elif self.items[bank][item]=="B": data = self.file.getBytes(bank,item)
+                        item_type = self.items[bank][item]
+                        if   item_type=="D": data = self.file.getDoubles(bank,item)
+                        elif item_type=="I": data = self.file.getInts(bank,item)
+                        elif item_type=="F": data = self.file.getFloats(bank,item)
+                        elif item_type=="L": data = self.file.getLongs(bank,item)
+                        elif item_type=="S": data = self.file.getShorts(bank,item)
+                        elif item_type=="B": data = self.file.getBytes(bank,item)
 
                         # Add bank data to batch dictionary
-                        if not bank+"_"+item in self.dict.keys() : self.dict[bank+"_"+item] = [data]
-                        else: self.dict[bank+"_"+item].append(data)            
+                        if not bank+"_"+item in self.dict: self.dict[bank+"_"+item] = [data]
+                        else: self.dict[bank+"_"+item].append(data)
 
                 # Check size of output array
                 self.counter += 1
@@ -958,7 +974,7 @@ class hipochainIterator:
             self.switchFile()
 
         # Final return for remainder
-        if self.dict != None:
+        if self.dict != None and len(self.dict.keys())>0:
             res       = self.dict
             self.dict = None
             return res #TODO: Will this return last remainder that is not necessarily stepsize?
